from neaty import log as LOG

from .config import ConfigRepo, Strategy
from .common import Args, UsageError

from inigrep.cli import Options


def run_conf_builtin(conf_repo: ConfigRepo,
                     args: list[str],
                     ) -> int:
    if not args:
        raise UsageError('no conf args?')
    a = Args(args)

    def has_kp():
        return '.' in a.peek2()[1]
    files = []
    ig_limit = None
    ig_query = None
    ig_mode = None
    strategy = Strategy.FIRST
    while a.rest() and ig_mode is None:
        if a.mpeek('-j'):
            strategy = Strategy.JOIN
            a.take1()
        elif a.mpeek('-1'):
            ig_limit = a.take1()
        elif a.mpeek('-e') and has_kp():
            ig_mode, ig_query = a.take2()
        elif a.mpeek('-r') and has_kp():
            ig_mode, ig_query = a.take2()
        elif a.mpeek('-K'):
            ig_mode, ig_query = a.take2()
        elif a.mpeek('-S'):
            ig_mode = a.take1()
        elif a.mpeek('-P'):
            ig_mode = a.take1()
        elif a.mpeek('-c'):
            ig_mode = a.take1()
        elif a.mpeek('-s'):
            ig_mode, ig_query = a.take2()
        elif a.peek().startswith('.'):
            raise UsageError(f"bad syntax: {a.args}")
        elif '.' in a.peek():
            ig_mode, ig_query = '-e', a.take1()
        elif a.mpeek('--help'):
            raise UsageError("helping...")
        else:
            raise UsageError(f"bad syntax: {a.args}")
    LOG.debugv('ig_limit', ig_limit)
    LOG.debugv('ig_query', ig_query)
    LOG.debugv('ig_mode', ig_mode)
    LOG.debugv('strategy', strategy)
    if ig_mode is None:
        raise UsageError(f"could not determine inigrep mode: {a.args}")
    if a.rest():
        files = a.rest()
    elif ig_query:
        files = [ig_query.split('.', 1)[0] + conf_repo.suffix]
    else:
        files = conf_repo.conf_filenames()
        if not files:
            LOG.warn("no conf files found")
    LOG.debugv('files', files)
    ig_args = [a for a in [ig_limit, ig_mode, ig_query] if a is not None]
    LOG.debugv('ig_args', ig_args)
    ig_opts = Options.from_args(ig_args)
    LOG.debugv('ig_opts', ig_opts)
    ig_gen = ig_opts.engine._r_run(conf_repo._load(files, strategy))
    es = 1
    for line in ig_gen:
        es = 0
        print(line)
    return es
